﻿// SoundLoop Stardew Valley Mod
// Copyright (C) 2022 strelkasaur
//
//     This program is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <https://www.gnu.org/licenses/>.

using System;
using Microsoft.Xna.Framework;
using StardewModdingAPI;
using StardewModdingAPI.Events;
using StardewModdingAPI.Utilities;
using StardewValley;
using StardewValley.Locations;


namespace SoundLoopMod
{
    public class ModEntry : Mod
    {
        public override void Entry(IModHelper helper)
        {
            helper.Events.GameLoop.OneSecondUpdateTicking += this.CheckMusicNeedsRestarting;
        }

        private void CheckMusicNeedsRestarting(object? sender, OneSecondUpdateTickingEventArgs e)
        {
            // Don't do anything if we're at the title screen
            if (!Context.IsWorldReady)
                return;
            
            // Music won't play again in the mines if this timestamp is lower than 150000. Set to some arbitrary large value.
            // The skull cavern is considered an extension of the mines, so this should affect that as well.
            MineShaft.timeSinceLastMusic = 999999;
            
            // Avoid playing morning song in the mines etc.
            if (!Game1.currentLocation.IsOutdoors)
                return;
        
            // The game keeps trying to stop the music if it's dark. We won't fight it for now
            if (Game1.isDarkOut())
                return;
            
            // Don't start music while on the bus
            // shouldHideCharacters seems to only be used for the bus, seems to be the nicest way to get this info
            if (Game1.currentLocation.shouldHideCharacters())
                return;
            
            // Be extra-safe about not doing anything during an event (to protect the special cases below)
            if (Game1.eventUp)
                return;
        
            // Seems to be safe to touch the current song, check if it has finished
            if (Game1.currentSong == null || Game1.currentSong.IsStopped || Game1.requestedMusicTrack.ToLower().Contains("ambient"))
            {
                if (Game1.currentLocation.Name == "Woods")
                {
                    // The music for the secret woods is a bit of a special case.
                    // Handle this in a way similar to the game when entering the woods.
                    
                    // The game doesn't start the woods song after 1800, do the same here
                    if (Game1.timeOfDay >= 1800)
                        return;
                    Game1.changeMusicTrack("woodsTheme");
                }
                else
                {
                    // General case, let the game figure out what song to play.
                    Game1.playMorningSong();
                }
                this.Monitor.Log("Restarted music");
            }
        }
    }
}
